@@ -1,16 +1,16 @@
 /*
- * SPDX-FileCopyrightText: 2020 Arjen Hiemstra <ahiemstra@heimr.nl>
- * 
- * SPDX-License-Identifier: LGPL-2.0-or-later
- */
+    SPDX-FileCopyrightText: 2020 Arjen Hiemstra <ahiemstra@heimr.nl>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #pragma once
 
 #include <QObject>
 #include <QRunnable>
 
-namespace KSysGuard {
-
+namespace KSysGuard
+{
 class ReadProcSmapsRunnable : public QObject, public QRunnable
 {
     Q_OBJECT
