@@ -44,6 +44,7 @@
 #include <KIO/ApplicationLauncherJob>
 #include <KService>
 #include <KWindowSystem>
+#include <KX11Extras>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
 
@@ -673,7 +674,7 @@ void KSysGuardProcessList::actionTriggered(QObject *object)
             QVariant widVar = d->mModel.data(realIndex, ProcessModel::WindowIdRole);
             if (!widVar.isNull()) {
                 int wid = widVar.toInt();
-                KWindowSystem::activateWindow(wid);
+                KX11Extras::activateWindow(wid);
             }
         }
     } else if (result == d->processDetails) {
@@ -1110,6 +1111,9 @@ void KSysGuardProcessList::updateList()
             updateFlags |= KSysGuard::Processes::IOStatistics;
         if (!d->mUi->treeView->isColumnHidden(ProcessModel::HeadingXMemory))
             updateFlags |= KSysGuard::Processes::XMemory;
+        // Updating VmPSS every call results in ~4x CPU load on my machine, so do it less often
+        if (!d->mUi->treeView->isColumnHidden(ProcessModel::HeadingVmPSS) && d->mResortCountDown <= 1)
+            updateFlags |= KSysGuard::Processes::Smaps;
         d->mModel.update(d->mUpdateIntervalMSecs, updateFlags);
         if (d->mUpdateTimer)
             d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
